/*************************************************************
*                           AD7864-1                         *
*                   Mikroprocesor: ATMEGA16                  *
*                     Napsal: Martin Samek                   *
*************************************************************/                

#include <avr/io.h> 
#include <avr/interrupt.h> 
#include "ad7864.h"
//----------------------< Glob. Prom >------------------------
unsigned int data[4];  // Vsledky pevodu
 
/*************************************************************
*                     ---< ad_init >---                      *
*                Inicializace rozhrani s AD                  *
*            Nutne zavolat pred prvnim pouzitim AD           *
*************************************************************/
void ad_init(void){
  DB0_DB7_DDR = 0x00;    		// Port na vstup
  DB8_DB11_DDR = 0xf0;   		// Port na vstup
  CS_DDR |= (1<<CS_DD);  		// Furt bude vystup
  WR_DDR |= (1<<WR_DD);  		// Furt bude vystup
  RD_DDR |= (1<<RD_DD);  		// Furt bude vystup
  CONVST_DDR |= (1<<CONVST);	// Furt bude vystup
  CS_PORT |= (1<<CS);    		// Klidova uroven je 1 
  WR_PORT |= (1<<WR);    		// Klidova uroven je 1
  RD_PORT |= (1<<RD);    		// Klidova uroven je 1
  CONVST_PORT |= (1<<CONVST);   // Klidova urovrn je 1
}
/*************************************************************
*                  ---< select_channel >---                  *
* Vbr kanlu pokud to nen provedeno hardwarov H/S = AGND *
*    Vstupem je vbr kanl pro pevod v souboru ad7864.h   *
*************************************************************/               
void select_channel(void){

unsigned char zpomocna=0;

  zpomocna = zpomocna + SL4;  // V dalsich par instrukcich prerovnam co mam odeslat
  zpomocna = zpomocna << 1;
  zpomocna = zpomocna + SL3;
  zpomocna = zpomocna << 1;
  zpomocna = zpomocna + SL2;
  zpomocna = zpomocna << 1;
  zpomocna = zpomocna + SL1;
  DB0_DB7_DDR = 0x0f;   // port na vystup 
  DB0_DB7_PORT = zpomocna;
  CS_PORT &= ~(1<<CS);  // CS 0
  WR_PORT &= ~(1<<WR);  // WR 0 -> zapis
  asm("nop");           // Mala prodleva
  WR_PORT |= (1<<WR);   // WR 1
  CS_PORT |= (1<<CS);   // CS 1
  DB0_DB7_DDR = 0x00;   // port na vstup
}


/************************************************************
*                   ---< start_convert >---                 *
*                      Sputn pevodu                     *
************************************************************/
void start_convert(void){
  CONVST_PORT &= ~(1<<CONVST);  // Dam do L
  asm("nop");					// Mala prodleva
  CONVST_PORT |= (1<<CONVST);   // Vzratim na H
}

/***********************************************************
*                     ---< get_data >---                   *
*          Do Glob. prom. se ulo vsledky pevodu        *
*        Ureno pro ten dat po pevodu vech kanl      *
*                 Vyuit ext. int. od BUSY                 *
***********************************************************/
void get_data(void){
  if(SL1 == 1)data[0] = read_port();  
  if(SL2 == 1)data[1] = read_port();  
  if(SL3 == 1)data[2] = read_port();  
  if(SL4 == 1)data[3] = read_port();  
}


/**********************************************************
*                     ---< read_port >---                 *
*                   Funkce pro ten portu                *
**********************************************************/
int read_port(void){

  unsigned int xdata=0;
  unsigned char xpomocna=0;  
 
    CS_PORT &= ~(1<<CS);  // CS 0
    asm("nop");           // Mala prodleva
    RD_PORT &= ~(1<<RD);  // RD 0 -> ctu
	asm("nop");           // Mala prodleva
	xdata = DB8_DB11_PIN; // Dalsich par instrukci preskladam data 
	xdata = xdata & 0x000f;
	xdata = xdata << 8;
    xpomocna =DB0_DB7_PIN; 
	xdata = xdata + xpomocna;
	RD_PORT |= (1<<RD);	 // Vratim ze nectu
 	CS_PORT |= (1<<CS);  // CS 1
	return xdata;
}
//------------------------< MAIN >-------------------------
int main(void){

//--------------< Nastaveni ext. preruseni >--------------
  DDRD &= ~(1<<DDD2);  	// externi int.
  GICR |=(1<<INT0);   	// Povolim ext. int0
  MCUCR &= ~(1<<ISC00);  // preruseni pri nizke urovni
  MCUCR |= (1<<ISC01);  // preruseni pri nizke urovni
  sei();  // povolim preruseni
//-------------------------------------------------------- 
 
  ad_init();   // inicializace AD prevodniku
  if(HS_SEL == 1){
    select_channel();
  }
  start_convert();
  while(1){
  }
}
//----------------------< INTERRUPT >--------------------
ISR (INT0_vect){
  get_data();      // Prectu data z AD
  if(HS_SEL == 1){
    select_channel();
  }
}
